'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = connectActionSets;

var _types = require('./types');

/**
 * Get a set of actions, connected to this store ready to dispatch to the reducers.
 *
 * @param  {String} actionSetID The unique identifier for the action set
 * @return {Object}             The connected action functions
 */
function getConnectedActionSet(_ref) {
  var actionSet = _ref.actionSet;
  var actionSetID = _ref.actionSetID;
  var getAllConnectedActionSets = _ref.getAllConnectedActionSets;
  var dispatch = _ref.dispatch;
  var getConsensusForIntrospection = _ref.getConsensusForIntrospection;

  var connectedActionSet = {
    getConsensus: function getConsensus() {
      throw "This action set has no introspection methods, maybe you have a typo.";
    }
  };

  Object.keys(actionSet).forEach(function (actionID) {
    if (actionID === _types.INTROSPECTION_TYPE) {
      var introspectionIDs = Object.keys(actionSet[_types.INTROSPECTION_TYPE]);
      connectedActionSet.getConsensus = introspectionIDs.reduce(function (consensusFunctions, introspectionID) {
        consensusFunctions[introspectionID] = getConsensusForIntrospection(introspectionID);
        return consensusFunctions;
      }, {});
      return;
    }

    var sourceActionFunction = actionSet[actionID];

    connectedActionSet[actionID] = function () {
      var payload = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      // Synchronous, immediately dispatched
      if (sourceActionFunction.length <= 1) {
        var result = sourceActionFunction(payload);
        // Allow empty action declarations
        if (typeof result !== 'undefined') {
          payload = result;
        }
        return dispatch({ actionSetID: actionSetID, actionID: actionID, payload: payload });
      }
      // Asychronous, delegates the dispatching
      else {
          return sourceActionFunction(payload, {
            currentActionSet: connectedActionSet,
            allActionSets: getAllConnectedActionSets()
          });
        }
    };
  });

  return connectedActionSet;
}

/**
 * Get multiple actions sets, connected to this store ready to dispatch to the reducers.
 *
 * @param  {Array} actionSetIDs The unique identifiers of the action sets.
 * @return {Object}             The action set identifiers to connected action function sets.
 */

function connectActionSets(_ref2) {
  var actionSets = _ref2.actionSets;
  var dispatch = _ref2.dispatch;
  var getConsensusForActionSet = _ref2.getConsensusForActionSet;

  return Object.keys(actionSets).reduce(function (connectedActionSets, actionSetID) {
    connectedActionSets[actionSetID] = getConnectedActionSet({
      actionSet: actionSets[actionSetID],
      actionSetID: actionSetID,
      getAllConnectedActionSets: function getAllConnectedActionSets() {
        return connectedActionSets;
      },
      dispatch: dispatch,
      getConsensusForIntrospection: getConsensusForActionSet(actionSetID)
    });

    return connectedActionSets;
  }, {});
}

module.exports = exports['default'];